#include"Esp8266Vars2File.h"

void setup(){
    Serial.begin(115200);
    delay(5000);


    // -----------var assignments
    char strc[14];
    strcpy(strc, "nopqrstuvwxyz");
    unsigned strc_len=strlen(strc);
    double THE_PI=3.1415926f;
    String str="abcdefghijklm";

    // since size of String always changes,
    // here uses String type for demo is merely a demo.
    // namely, the cDataBase not applies to dynamic objects(classes do mem managements/nongranulated objs).
    unsigned str_length=sizeof(str);
    unsigned tmp_string[(str_length+3)/4]; //// for alignment.
    char *new_string=(char*)tmp_string; //// let new_string to fit String type.


    // -----------Open database file, auto create new if not exist
    cDataBase s;
    s="/database";
    s.Open();


    printf("3 vars are, pi=%f, str=%s, strc=%s\r\n", THE_PI, str.c_str(), strc);


    // -----------store vars to db file
    s.WriteBytes("ThisIsChars", 99, strc, strc_len, strc_len);
    s.WriteWords("doubleTHE_PI", 99, THE_PI);
    s.WriteBytes("ThisIsAString1", 99, &str, 1, 1);
    s.WriteBytes("ThisIsAString2", 99, (unsigned char*)&str, sizeof(str), sizeof(str));
    s.WriteWords("[KeyWordHere]ThisIsADesignatorToPointOutTheId99IsWrittenAtTimestamp", 20210321, 99);


    // -----------take a look at the db file
    printf("the context of the database file is as follows(in ascii):\r\n");
    s.DumpAll(0);
    printf("the context of the database file is as follows(in hex):\r\n");
    s.DumpAll(1);
    printf("the records in the database file are as follows:\r\n");
    s.LsRecords();


    // -----------zeroing the vars
    THE_PI=0;
    str="";
    strc[0]=0;
    printf("zeroing the vars, pi=%f, str=%s, strc=%s\r\n", THE_PI, str.c_str(), strc);


    // -----------restore the vars from db file
    s.ReadBytes(strc, strc_len, "ThisIsChars");

    s.ReadWords(THE_PI, "doubleTHE_PI");

    ////s.ReadBytes(&str, str_length, "ThisIsAString1"); //// don't do that.
    s.ReadBytes(new_string, str_length, "ThisIsAString1"); //// use this instead.
    str=*(reinterpret_cast<String*>(new_string)); //// not work. just for demo.

    printf("after restore, pi=%f, str=%s, strc=%s\r\n", THE_PI, str.c_str(), strc);

    s.ReadBytes(new_string, str_length, "ThisIsAString2"); //// use this instead.
    str=*(reinterpret_cast<String*>(new_string)); //// not work. just for demo.
    printf("str=%s, str=%s\r\n", str.c_str(), (static_cast<String>(*new_string)).c_str());

    int a, b;
    s.ReadWords(a, b, "[KeyWordHere]ThisIsADesignatorToPointOutTheId99IsWrittenAtTimestamp");
    printf("timestamp=%d, id=%d\r\n", b, a);


    // -----------add more records, then show the same id and find the max id of "ThisIsChars"
    s.WriteBytes("ThisIsChars", 98, strc, strc_len, strc_len);
    s.WriteBytes("ThisIsChars", 97, strc, strc_len, strc_len);
    s.WriteBytes("ThisIsChars", 99, strc, strc_len, strc_len);
    s.WriteBytes("ThisIsChars", 199, strc, strc_len, strc_len);
    s.WriteBytes("ThisIsChars", 299, strc, strc_len, strc_len);
    s.WriteBytes("ThisIsChars", 96, strc, strc_len, strc_len);
    s.PrintIdEx(99);
    printf("Max id of ThisIsChars is %d\r\n", s.FindMaxIdEx("ThisIsChars"));


    // -----------replace ["ThisIsChars", id=299] var with data "abcdef0123456789"+10 times "abc"+"z".
    s.Close();
    if (s.ReplaceBytesEx("ThisIsChars", 299, "ab", strlen("ab"), strlen("abcdef0123456789")+10*strlen("abc")+strlen("z"))){
        s.ReplaceBytesContEx("cd", strlen("cd"));
        s.ReplaceBytesContEx("ef0123456789", strlen("ef0123456789"));
        for (int i=0; i<10; i++) s.ReplaceBytesContEx("abc", strlen("abc"));
        s.ReplaceBytesContEx("z", strlen("z"));
    }
    s.DumpAll(0);
    s.LsRecords();


    // -----------read ["ThisIsChars", id=299] var by ReadData
    printf("the ThisIsChars, id=299 contents:\r\n");
    s.Open();
    unsigned char buff[5];

    int d=s.ReadData(buff, 4, "ThisIsChars", 299);

    if (d<=4){ // finished
        buff[d]=0;
        printf("%s\r\n", (char*)buff);
    }
    else {
        int readed=4;
        d-=readed;
        while (1){
            buff[readed]=0;
            printf("%s", (char*)buff);
            if (d<4) break;
            readed=s.ReadDataCont(buff, 4);
            d-=readed;
        } // after, remaining 3 bytes at most unread.
        do {
            d-=(readed=s.ReadDataCont(buff, d));
            buff[readed]=0;
            printf("%s", (char*)buff);
        } while (d);
        printf("%c%c", '\r', '\n');
    }


    // -----------show the Cp
    s.Close();
    cDataBase::Cp("/database", "/default");

    s="/default";
    s.LsRecords();
    s.DumpAll(0);

    s="/database";
    s.LsRecords();
    s.DumpAll(0);


    // -----------delete a record in db file
    s.Close();
    s.DeleteRecords("doubleTHE_PI");
    printf("after delete a record, the records in the db file are:\r\n");
    s.LsRecords();

    printf("after delete a record, the context(unchanged) of the database file(hex):\r\n");
    s.DumpAll(1);


    // -----------compact the db file
    s.Close();
    s.Renew();
    printf("after renew, the context(shrinked) of the database file(hex):\r\n");
    s.DumpAll(1);

    cDataBase::Ls("/");
}

void loop(){
}
